/****************************************************************************
 * (C) Nicolas GEORGE 1999                                                  *
 * Ce programme fait partie du projet xbackground.                          *
 * Il est distribu sous les termes de la GNU-GPL, dont un exemplaire doit  *
 * tre joint (si ce n'est pas le cas, vous le trouverez sur www.fsf.org).  *
 * Je ne fournis aucune garantie sur ce programme.                          *
 ****************************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "xbackground.h"

static int minr=32768,minv=32768,minb=32768;
static int maxr=65535,maxv=65535,maxb=65535;
static int dl=8;

#define ramp(i) (rand()%(((i)<<3)+1)-((i)<<2))
#define pla(x,y) (t[((x)&lm)+(((y)&lm)<<dl)])

static int lire_chaine(char *c,char *v)
{
  if((!strcmp(c,"dark"))||(!strcmp(c,"light"))){
    XColor co;

    if(!XParseColor(disp,colormap,v,&co)){
      fprintf(stderr,"Invalid color: %s\n",v);
      return(-1);
    }
    if(!strcmp(c,"dark")){
      minr=co.red;
      minv=co.green;
      minb=co.blue;
    }else{
      maxr=co.red;
      maxv=co.green;
      maxb=co.blue;
    }
    return(1);
  }else if(!strcmp(c,"size")){
    dl=atoi(v);
    if(dl<3) dl=3;
    if(dl>12) dl=12;
    return(1);
  }
  return(0);
}

static int initialiser(int *l,int *h)
{
  int i,r,v,b;

  if(*l>1<<dl) *l=1<<dl;
  if(*h>1<<dl) *h=1<<dl;

  for(i=0;i<nb_col;i++){
    r=minr+(i*(maxr-minr))/(nb_col-1);
    r=r<0?0:r>65535?65535:r;
    v=minv+(i*(maxv-minv))/(nb_col-1);
    v=v<0?0:v>65535?65535:v;
    b=minb+(i*(maxb-minb))/(nb_col-1);
    b=b<0?0:b>65535?65535:b;
    pal[i]=((long)(r/257))|(((long)(v/257))<<8)|(((long)(b/257))<<16);
  }
  return(0);
}

static void creer_plasma(int ls,int hs,XImage *img)
{
  int l,i,x,y,lm,i2;
  short *t;

  l=1<<dl;
  lm=l-1;
  t=malloc(sizeof(short)*l*l);
  t[0]=0;
  for(i=l;i>1;i>>=1){
    i2=i>>1;
    for(x=0;x<l;x+=i)
      for(y=0;y<l;y+=i){
	pla(x,y+i2)=((pla(x,y)+pla(x,y+i))>>1)+ramp(i);
	pla(x+i2,y)=((pla(x,y)+pla(x+i,y))>>1)+ramp(i);
      }
    for(x=0;x<l;x+=i)
      for(y=0;y<l;y+=i)
	pla(x+i2,y+i2)=((pla(x,y)+pla(x+i,y)+pla(x,y+i)+pla(x+i,y+i))>>2)+ramp(i);

    for(x=i2;x<l;x+=i)
      for(y=i2;y<l;y+=i){
	pla(x,y+i2)=((pla(x,y)+pla(x,y+i))>>1)+ramp(i);
	pla(x+i2,y)=((pla(x,y)+pla(x+i,y))>>1)+ramp(i);
      }
    for(x=i2;x<l;x+=i)
      for(y=i2;y<l;y+=i)
	pla(x+i2,y+i2)=((pla(x,y)+pla(x+i,y)+pla(x,y+i)+pla(x+i,y+i))>>2)+ramp(i);
  }

  for(y=0;y<hs;y++)
    for(x=0;x<ls;x++){
      i=128+((pla(x+1,y+1)-pla(x,y))<<2);
      if(i<0) i=0;
      if(i>255) i=255;
      XPutPixel(img,x,y,pal[i*(nb_col-1)/255]);
    }
  free(t);
}

moteur_de_fond moteur_plasma={
  "plasma",
  lire_chaine,
  initialiser,
  creer_plasma
};
